#!/bin/bash

### Version
VERSION="1.10.1"

### Namespace
ORIG_NAME="dxreminders"
PRODUCT_NAME="dxreminders"
DMG_NAME="${PRODUCT_NAME}-${VERSION}"

### Environment
SVN_ROOT=".."

# Directories
TMP_DIR="Applications"
SCRIPT_DIR=`pwd`/`dirname $0`

### DMG part

cd $SCRIPT_DIR
TMP_DMG_NAME="temporary.dmg"
if [ -f $TMP_DMG_NAME ]
then
    rm $TMP_DMG_NAME
fi

echo "Creating temporary dmg disk image..."
hdiutil create -ov -srcfolder $TMP_DIR -format UDRW -volname "$PRODUCT_NAME" "$TMP_DMG_NAME"

echo "Mounting temporary image..."
for i in /Volumes/${PRODUCT_NAME}*
do
    if [[ -d $i ]]
    then
        hdiutil unmount "$i"
    fi
done
device=$(hdiutil attach -readwrite -noverify -noautoopen "$TMP_DMG_NAME" | egrep '^/dev/' | sed 1q | awk '{print $1}')
echo "done! (device ${device})"

echo "Sleeping for 5 seconds..."
sleep 5

echo "Setting style for temporary dmg image..."
echo "* Copying background image... "
BG_FOLDER="/Volumes/$PRODUCT_NAME/.background"
mkdir "$BG_FOLDER"
cp "$SCRIPT_DIR/backround.png" "${BG_FOLDER}/"
echo "* Copying volume icon... "
ICON_FOLDER="/Volumes/$PRODUCT_NAME"
cp "$SVN_ROOT/data/icons/dxreminders.icns" "$ICON_FOLDER/.VolumeIcon.icns"
echo "* Setting volume icon... "
SetFile -c icnC "$ICON_FOLDER/.VolumeIcon.icns"
SetFile -a C "$ICON_FOLDER"
echo "* Adding symlink to /Applications... "
ln -s /Applications "/Volumes/$PRODUCT_NAME/Applications"
echo "done!"

APPS_X=380
APPS_Y=185
BUNDLE_X=110
BUNDLE_Y=185
WINDOW_WIDTH=500
WINDOW_HEIGHT=300
WINDOW_LEFT=300
WINDOW_TOP=100
WINDOW_RIGHT=$(($WINDOW_LEFT + $WINDOW_WIDTH))
WINDOW_BOTTOM=$(($WINDOW_TOP + $WINDOW_HEIGHT))

echo "Executing applescript for further customization..."
APPLESCRIPT="
tell application \"Finder\"
    tell disk \"$PRODUCT_NAME\"
        open
        -- Setting view options
        set current view of container window to icon view
        set toolbar visible of container window to false
        set statusbar visible of container window to false
        set the bounds of container window to {${WINDOW_LEFT}, ${WINDOW_TOP}, ${WINDOW_RIGHT}, ${WINDOW_BOTTOM}}
        set theViewOptions to the icon view options of container window
        set arrangement of theViewOptions to not arranged
        set icon size of theViewOptions to 96
        -- Settings background
        ${NO_BG}set background picture of theViewOptions to file \".background:backround.png\"
        -- Rearranging
        set the position of item \"Applications\" to {$APPS_X, $APPS_Y}
        set the position of item \"$PRODUCT_NAME\" to {$BUNDLE_X, $BUNDLE_Y}
        -- Updating and sleeping for 5 secs
        update without registering applications
        -- Reopening
        close
        open
        delay 6
        -- Reopening, for .DS_STORE
        close
        open
        delay 6
        -- Eject
        eject
    end tell
end tell
"
echo "$APPLESCRIPT" | osascript
echo "done!"

echo "Fixing permissins and syncing..."
chmod -Rf go-w /Volumes/"$PRODUCT_NAME" &> /dev/null || true
sync
echo "done!"

echo "Converting..."
if [ -f $DMG_NAME ]
then
    rm $DMG_NAME
fi
hdiutil convert "$TMP_DMG_NAME" -format UDBZ -o "$SCRIPT_DIR/$DMG_NAME"
echo "done!"

echo "Removing temporary dmg disk image..."
rm $TMP_DMG_NAME
echo "done!"

echo "Everything done. DMG disk image is ready for distribution."
