/***************************************************************
 * Name:      engine.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#ifndef ENGINE_H
#define ENGINE_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <vector>
#include "wx/wxsqlite3.h"
#include "event.h"
#include "recurrence.h"

class Engine
{
    public:
        Engine() {}
        virtual ~Engine();

        bool Open(const wxString &filename);
        void LoadData();
        Event AddEvent(const wxString &event, const wxString &descr, const wxDateTime &date, const wxLongLong &reminder);
        void EditEvent(long id, const wxString &event, const wxString &descr, const wxDateTime &date, const wxLongLong &reminder);
        void EditEvent(long id, const wxLongLong &reminder);
        void RemoveEvent(long id);
        void RecurrentEvent(long id);
        long GetEventsCount() const { return static_cast<long>(m_events.size()); }
        wxString GetItemText(long item, long column) const;
        Event GetEvent(long index);
        Event GetEventByID(long id);
        std::vector<Event> GetEventsByReminder(wxDateTime time);
        void SetEventReminded(long id, bool reminded = true);
        /**
        * @param id     Event ID
        */
        bool HasRecurrence(long id);
        Recurrence GetRecurrenceByEventID(long eventid);
        Recurrence AddRecurrence(int type, int days, int months, long eventid);
        /**
        * @param id     Event ID
        */
        void RemoveRecurrence(long id);
        /**
        * @param id     Event ID
        */
        void EditRecurrence(long id, int type, int days, int months);
    private:
        wxSQLite3Database m_db;
        std::vector<Event> m_events;
        std::vector<Recurrence> m_recurrences;

        void CreateEventsTable();
        void CreateRecurrenceTable();
        size_t GetEventIndexByID(long id);
        /**
        * @param id     Event ID
        */
        size_t GetRecurrenceIndexByEventID(long id);
};

#endif // ENGINE_H
