/***************************************************************
 * Name:      dxreminders.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include <wx/aboutdlg.h>
#include <wx/taskbar.h>
#include <wx/listctrl.h>
#include <wx/stdpaths.h>
#include <wx/sysopt.h>

#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__)
    #include "../data/icons/dxreminders.xpm"
#endif
#include "dxdefs.h"
#include "config.h"
#include "dxreminders.h"
#include "eventdialog.h"
#include "eventmsgdialog.h"

dxremindersFrame* frame = NULL;
bool frameShown = true;

IMPLEMENT_APP(dxremindersApp)

bool dxremindersApp::OnInit()
{
#if defined (__WXMAC__) || defined (__WXOSX__)
    wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"),1);
#endif
    wxLocale::AddCatalogLookupPathPrefix(_T(LOCALEDIR));
    m_Locale = new wxLocale();
    m_Locale->Init(wxLANGUAGE_DEFAULT);
    m_Locale->AddCatalog(_T("dxreminders"));
    m_Locale->AddCatalog(_T("wxstd"));

    wxSQLite3Database::InitializeSQLite();

    frame = new dxremindersFrame(0L, _T("dxReminders"));
    frame->SetIcon(wxICON(dxreminders)); // To Set App Icon
    frame->Show();
    frame->LoadDatabase();
    return true;
}

int dxremindersApp::OnExit()
{
    wxSQLite3Database::ShutdownSQLite();
    return 0;
}

BEGIN_EVENT_TABLE(dxTaskBarIcon, wxTaskBarIcon)
    EVT_TASKBAR_LEFT_DCLICK(dxTaskBarIcon::OnLeftButtonDClick)
    EVT_MENU(ID_TASK_RESTORE, dxTaskBarIcon::OnRestore)
    EVT_MENU(ID_MENU_NEWEVENT, dxTaskBarIcon::OnNewEvent)
    EVT_MENU(wxID_EXIT, dxTaskBarIcon::OnQuit)
END_EVENT_TABLE()

void dxTaskBarIcon::OnLeftButtonDClick(wxTaskBarIconEvent&)
{
    if(frameShown)
        frame->Show(false);
    else
    {
        frame->Show(true);
        frame->Iconize(false);
    }
    frameShown = !frameShown;
}

wxMenu *dxTaskBarIcon::CreatePopupMenu()
{
    wxMenu *menu = new wxMenu;
    if(frameShown)
        menu->Append(ID_TASK_RESTORE, _("Hide"));
    else
        menu->Append(ID_TASK_RESTORE, _("Restore"));
    menu->AppendSeparator();
    menu->Append(ID_MENU_NEWEVENT, _("New event"));
#if !(defined (__WXMAC__) || defined (__WXOSX__)) /*Mac has built-in quit menu*/
    menu->AppendSeparator();
    menu->Append(wxID_EXIT);
#endif
    return menu;
}

void dxTaskBarIcon::OnRestore(wxCommandEvent& /*event*/)
{
    if(frameShown)
        frame->Show(false);
    else
    {
        frame->Show(true);
        frame->Iconize(false);
    }
    frameShown = !frameShown;
}

void dxTaskBarIcon::OnQuit(wxCommandEvent& event)
{
    wxPostEvent(frame, event);
}

void dxTaskBarIcon::OnNewEvent(wxCommandEvent& event)
{
    wxPostEvent(frame, event);
}

wxString EventListCtrl::OnGetItemText(long item, long column) const
{
    return frame->getDb()->GetItemText(item, column);
}

int EventListCtrl::OnGetItemImage(long /*item*/) const
{
    return 0;
}

wxListItemAttr *EventListCtrl::OnGetItemAttr(long /*item*/) const
{
    return NULL;
}

BEGIN_EVENT_TABLE(dxremindersFrame, wxFrame)
    EVT_CLOSE(dxremindersFrame::OnClose)
    EVT_MENU(wxID_EXIT, dxremindersFrame::OnQuit)
    EVT_MENU(wxID_ABOUT, dxremindersFrame::OnAbout)
    EVT_MENU(ID_MENU_NEWEVENT, dxremindersFrame::OnNewEvent)
    EVT_MENU(ID_MENU_EDITEVENT, dxremindersFrame::OnEditEvent)
    EVT_MENU(ID_MENU_REMOVEEVENT, dxremindersFrame::OnRemoveEvent)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_EVENTLIST, dxremindersFrame::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_EVENTLIST, dxremindersFrame::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_EVENTLIST, dxremindersFrame::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_EVENTLIST, dxremindersFrame::OnItemDeselected)
    EVT_TIMER(wxID_ANY, dxremindersFrame::OnTimer)
END_EVENT_TABLE()

dxremindersFrame::dxremindersFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxSize(500,250)), m_selectedIndex(-1), m_timer(this)
{
#if wxUSE_MENUS
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu();
#if defined (__WXMAC__) || defined (__WXOSX__)
    fileMenu->Append(wxID_ABOUT);
#endif
    fileMenu->Append(ID_MENU_NEWEVENT, _("New event\tCtrl-N"));
    fileMenu->AppendSeparator();
    fileMenu->Append(wxID_EXIT);
    mbar->Append(fileMenu, _("&Events"));

#if !(defined (__WXMAC__) || defined (__WXOSX__))
    wxMenu* helpMenu = new wxMenu();
    helpMenu->Append(wxID_ABOUT);
    mbar->Append(helpMenu, _("&Help"));
#endif

    SetMenuBar(mbar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
    CreateStatusBar(1);
    SetStatusText(_T("dxReminders"),0);
#endif // wxUSE_STATUSBAR

#if defined(__WXCOCOA__)
    m_dockIcon = new dxTaskBarIcon(wxTaskBarIcon::DOCK);
#else
    m_taskBarIcon = new dxTaskBarIcon();
    m_taskBarIcon->SetIcon(wxICON(dxreminders));
#endif

    m_eventsList = new EventListCtrl(this, ID_EVENTLIST);
    m_eventsList->InsertColumn(0, _("Event"));
    m_eventsList->InsertColumn(1, _("Date"));
    m_eventsList->InsertColumn(2, _("Reminder"));
    m_eventsList->setResizeColumn(0);
    m_eventsList->SetColumnWidth(1, 150);
    m_eventsList->SetColumnWidth(2, 150);

    m_timer.Start(1000);
}

dxremindersFrame::~dxremindersFrame()
{
    delete m_taskBarIcon;
    if(m_timer.IsRunning())
        m_timer.Stop();
}

void dxremindersFrame::LoadDatabase()
{
    m_data = new Engine();
    wxString path = wxStandardPaths::Get().GetUserDataDir();
    if(!wxDirExists(path))
    {
        wxMkdir(path);
    }
    path << wxFILE_SEP_PATH << _T("reminders.db");
    if(!m_data->Open(path))
        wxMessageBox(wxString::Format(_("Couldn't open database file\n%s"), path.c_str()), _T(PACKAGE));
    else
    {
        m_data->LoadData();
        m_eventsList->SetItemCount(m_data->GetEventsCount());
    }
    FireEvents();
}

void dxremindersFrame::OnClose(wxCloseEvent& /*event*/)
{
    Show(false);
    frameShown = !frameShown;
}

void dxremindersFrame::OnQuit(wxCommandEvent& /*event*/)
{
    Destroy();
}

void dxremindersFrame::OnAbout(wxCommandEvent& /*event*/)
{
    wxString description = wxString() << wxVERSION_STRING << _T("\n")
                            << wxSQLite3Database::GetWrapperVersion() << _T("\n")
                            << _T("SQLite3 ") << wxSQLite3Database::GetVersion();
    wxAboutDialogInfo info;
    info.SetName(_T(PACKAGE));
    info.SetVersion(_T(VERSION));
    info.SetDescription(description);
    info.SetCopyright(_T("(c) 2013 ~"));
    info.SetWebSite(_T("http://dxreminders.dxsolutions.org"));
    info.AddDeveloper(_T("David Vachulka <arch_dvx@users.sourceforge.net>"));
    wxAboutBox(info);
}

void dxremindersFrame::OnNewEvent(wxCommandEvent& /*event*/)
{
    EventDialog dialog(this, m_data);
    if(dialog.ShowModal() == wxID_OK)
    {
        long cnt = m_data->GetEventsCount();
        m_eventsList->SetItemCount(cnt);
        if(cnt) m_eventsList->RefreshItems(0, cnt > 0 ? --cnt : 0);
        else m_eventsList->Refresh();
    }
}

void dxremindersFrame::OnEditEvent(wxCommandEvent& /*event*/)
{
    if(m_selectedIndex == -1) return;
    EventDialog dialog(this, m_data, m_data->GetEvent(m_selectedIndex));
    if(dialog.ShowModal() == wxID_OK)
    {
        long cnt = m_data->GetEventsCount();
        m_eventsList->SetItemCount(cnt);
        if(cnt) m_eventsList->RefreshItems(0, cnt > 0 ? --cnt : 0);
        else m_eventsList->Refresh();
    }
}

void dxremindersFrame::OnRemoveEvent(wxCommandEvent& /*event*/)
{
    if(m_selectedIndex == -1) return;
    wxMessageDialog dialog(this, _("Do you really want to delete the Event?"), _("Confirm Event Deletion"), wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_YES)
    {
        m_data->RemoveEvent(m_data->GetEvent(m_selectedIndex).GetID());
        long cnt = m_data->GetEventsCount();
        m_eventsList->SetItemCount(cnt);
        if(cnt) m_eventsList->RefreshItems(0, cnt > 0 ? --cnt : 0);
        else m_eventsList->Refresh();
    }
}

void dxremindersFrame::OnItemRightClick(wxListEvent& event)
{
    m_selectedIndex = event.GetIndex();
    wxMenu popup;
    popup.Append(ID_MENU_NEWEVENT, _("New event"));
    if(m_selectedIndex != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_MENU_EDITEVENT, _("Edit event"));
        popup.Append(ID_MENU_REMOVEEVENT, _("Remove event"));
    }
    PopupMenu(&popup, event.GetPoint());
}

void dxremindersFrame::OnItemActivated(wxListEvent& event)
{
    m_selectedIndex = event.GetIndex();
    EventDialog dialog(this, m_data, m_data->GetEvent(m_selectedIndex));
    if(dialog.ShowModal() == wxID_OK)
    {
        long cnt = m_data->GetEventsCount();
        m_eventsList->SetItemCount(cnt);
        m_eventsList->RefreshItems(0, cnt > 0 ? --cnt : 0);
    }
}

void dxremindersFrame::OnItemSelected(wxListEvent& event)
{
    m_selectedIndex = event.GetIndex();
}

void dxremindersFrame::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selectedIndex = -1;
}

void dxremindersFrame::OnTimer(wxTimerEvent& /*event*/)
{
    if(wxDateTime::Now().GetSecond() != 0) return;
    FireEvents();
}

void dxremindersFrame::FireEvents()
{
    std::vector<Event> events = m_data->GetEventsByReminder(wxDateTime::Now());
    if(events.size())
    {
        long id = -1;
        for(size_t i=0; i<events.size(); i++)
        {
            m_data->SetEventReminded(events[i].GetID());
            EventMsgDialog dialog(this, m_data, events[i]);
            if(dialog.ShowModal() == wxID_OK)
            {
                id = events[i].GetID();
                if(m_data->HasRecurrence(id)) m_data->RecurrentEvent(id);
                else m_data->RemoveEvent(id);
            }
            long cnt = m_data->GetEventsCount();
            m_eventsList->SetItemCount(cnt);
            if(cnt) m_eventsList->RefreshItems(0, cnt > 0 ? --cnt : 0);
            else m_eventsList->Refresh();
        }
    }
}

