/***************************************************************
 * Name:      dxreminders.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#ifndef DXREMINDERS_H
#define DXREMINDERS_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include "data/engine.h"
#include "widgets/dxlistctrl.h"

class dxremindersApp : public wxApp
{
    public:
        virtual bool OnInit();
        virtual int OnExit();
    protected:
        wxLocale *m_Locale;
};

class dxTaskBarIcon: public wxTaskBarIcon
{
public:
#if defined(__WXCOCOA__)
    dxTaskBarIcon(wxTaskBarIconType iconType = DEFAULT_TYPE) : wxTaskBarIcon(iconType)
#else
    dxTaskBarIcon()
#endif
    {}

    void OnLeftButtonDClick(wxTaskBarIconEvent&);
    virtual wxMenu *CreatePopupMenu();
private:
    void OnRestore(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);
    void OnNewEvent(wxCommandEvent& event);
DECLARE_EVENT_TABLE()
};

class EventListCtrl: public dxListCtrl
{
public:
    EventListCtrl(wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id) {}
private:
    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemImage(long item) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class dxremindersFrame: public wxFrame
{
public:
    dxremindersFrame(wxFrame *frame, const wxString& title);
    ~dxremindersFrame();

    void LoadDatabase();
    Engine *getDb() { return m_data; }
private:
    dxTaskBarIcon *m_taskBarIcon;
    EventListCtrl *m_eventsList;
    Engine *m_data;
    long m_selectedIndex;
    wxTimer m_timer;

    void OnClose(wxCloseEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);
    void OnNewEvent(wxCommandEvent& event);
    void OnEditEvent(wxCommandEvent& event);
    void OnRemoveEvent(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnTimer(wxTimerEvent& event);

    //fire events which have to be reminded
    void FireEvents();
DECLARE_EVENT_TABLE()
};


#endif // DXREMINDERS_H
