/***************************************************************
 * Name:      eventdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "dxdefs.h"
#include "eventdialog.h"

IMPLEMENT_DYNAMIC_CLASS(ReminderDialog, wxDialog)

ReminderDialog::ReminderDialog(wxWindow* parent, wxLongLong reminder)
: wxDialog(parent, wxID_ANY, wxString(_("Reminder time")))
{
    long mins = reminder.ToLong();
    long days = mins/(60*24);
    mins -= days*60*24;
    long hours = mins/60;
    mins -= hours*60;

    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *rmdSizer = new wxBoxSizer(wxHORIZONTAL);
    rmdSizer->Add(new wxStaticText(this, wxID_ANY, _("Days:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    m_days = new wxSpinCtrl(this);
    m_days->SetValue(days);
    rmdSizer->Add(m_days);
    rmdSizer->Add(new wxStaticText(this, wxID_ANY, _("Hours:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    m_hours = new wxSpinCtrl(this);
    m_hours->SetRange(0,23);
    m_hours->SetValue(hours);
    rmdSizer->Add(m_hours);
    rmdSizer->Add(new wxStaticText(this, wxID_ANY, _("Minutes:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    m_mins = new wxSpinCtrl(this);
    m_mins->SetRange(0,59);
    m_mins->SetValue(mins);
    rmdSizer->Add(m_mins);
    mainSizer->Add(rmdSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(mainSizer);
    mainSizer->Fit(this);
    mainSizer->SetSizeHints(this);
}

wxLongLong ReminderDialog::GetReminder()
{
    return m_days->GetValue()*60*24+m_hours->GetValue()*60+m_mins->GetValue();
}

IMPLEMENT_DYNAMIC_CLASS(RecurrenceDialog, wxDialog)

RecurrenceDialog::RecurrenceDialog(wxWindow* parent, int days, int months)
: wxDialog(parent, wxID_ANY, wxString(_("Recurrence")))
{
    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *rmdSizer = new wxBoxSizer(wxHORIZONTAL);
    rmdSizer->Add(new wxStaticText(this, wxID_ANY, _("Months:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    m_moths = new wxSpinCtrl(this);
    m_moths->SetValue(months);
    rmdSizer->Add(m_moths);
    rmdSizer->Add(new wxStaticText(this, wxID_ANY, _("Days:")), 0, wxALIGN_CENTER_VERTICAL|wxALL, 1);
    m_days = new wxSpinCtrl(this);
    m_days->SetValue(days);
    rmdSizer->Add(m_days);
    mainSizer->Add(rmdSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(mainSizer);
    mainSizer->Fit(this);
    mainSizer->SetSizeHints(this);
}

int RecurrenceDialog::GetDays()
{
    return m_days->GetValue();
}

int RecurrenceDialog::GetMonths()
{
    return m_moths->GetValue();
}

IMPLEMENT_DYNAMIC_CLASS(EventDialog, wxDialog)

BEGIN_EVENT_TABLE(EventDialog, wxDialog)
    EVT_BUTTON(wxID_OK, EventDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, EventDialog::OnCancel)
    EVT_CHOICE(ID_REMINDERCHOICE, EventDialog::OnReminder)
    EVT_CHOICE(ID_RECURRENCECHOICE, EventDialog::OnRecurrence)
END_EVENT_TABLE()

EventDialog::EventDialog(wxWindow *parent, Engine *db)
: wxDialog(parent, wxID_ANY, wxString(_("New event"))), m_editing(false), m_db(db), m_id(-1), m_reminder(0), m_type(0), m_days(0), m_months(0)
{
    CreateControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

EventDialog::EventDialog(wxWindow *parent, Engine *db, Event event)
: wxDialog(parent, wxID_ANY, wxString(_("Edit event"))), m_editing(true), m_db(db), m_id(event.GetID()), m_type(0), m_days(0), m_months(0)
{
    CreateControls();
    FillControls(event);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void EventDialog::CreateControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Event:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_eventCtrl = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_eventCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Description:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_descriptionCtrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(250, 50), wxTE_MULTILINE | wxHSCROLL);
    gridSizer->Add(m_descriptionCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_dateCtrl = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Now(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_dateCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Time:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_timeCtrl = new wxTimePickerCtrl(this, wxID_ANY, RoundTime());
    gridSizer->Add(m_timeCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    m_reminderText = new wxStaticText(this, wxID_ANY, _("Reminder:"));
    gridSizer->Add(m_reminderText, wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    wxArrayString choices;
    choices.Add(_("0 minutes before"));
    choices.Add(_("5 minutes before"));
    choices.Add(_("15 minutes before"));
    choices.Add(_("1 hour before"));
    choices.Add(_("2 hours before"));
    choices.Add(_("1 day before"));
    choices.Add(_("2 days before"));
    choices.Add(_("Own"));
    m_reminderCtrl = new wxChoice(this, ID_REMINDERCHOICE, wxDefaultPosition, wxDefaultSize, choices);
    m_reminderCtrl->SetSelection(0);
    gridSizer->Add(m_reminderCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Recurrence:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    wxArrayString recurrences;
    recurrences.Add(_("Never"));
    recurrences.Add(_("Daily"));
    recurrences.Add(_("Weekly"));
    recurrences.Add(_("Every two weeks"));
    recurrences.Add(_("Monthly"));
    recurrences.Add(_("Own"));
    m_recurrenceCtrl = new wxChoice(this, ID_RECURRENCECHOICE, wxDefaultPosition, wxDefaultSize, recurrences);
    m_recurrenceCtrl->SetSelection(0);
    gridSizer->Add(m_recurrenceCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void EventDialog::FillControls(Event event)
{
    m_eventCtrl->SetValue(event.GetEvent());
    m_descriptionCtrl->SetValue(event.GetDescr());
    m_dateCtrl->SetValue(event.GetDate());
    m_timeCtrl->SetValue(event.GetDate());
    m_reminder = event.GetReminder();
    if(m_reminder.ToLong() < 0)
    {
        m_reminderText->Hide();
        m_reminderCtrl->Hide();
        return;
    }
    int id;
    switch(m_reminder.ToLong())
    {
        case 0: id=0; break; //0 mins
        case 5: id=1; break; //5 mins
        case 15: id=2; break; //15 mins
        case 60: id=3; break; //1 hours
        case 120: id=4; break; //2 hours
        case 1440: id=5; break; //1 day
        case 2880: id=6; break; //2 days
        default:
        {
            id=7;
            long mins = m_reminder.ToLong();
            long days = mins/(60*24);
            mins -= days*60*24;
            long hours = mins/60;
            mins -= hours*60;
            m_reminderCtrl->SetString(7, wxString::Format(_("Own (%ld days, %ld hours, %ld minutes)"), days, hours, mins));
            break;
        }
    }
    m_reminderCtrl->SetSelection(id);
    Recurrence rec = m_db->GetRecurrenceByEventID(m_id);
    if(rec.IsValid())
    {
        m_type = rec.GetType();
        m_days = rec.GetDays();
        m_months = rec.GetMonths();
        if(m_type == R_OWN)
        {
            m_recurrenceCtrl->SetString(m_type, wxString::Format(_("Own (%d months, %d days)"), m_months, m_days));
        }
        m_recurrenceCtrl->SetSelection(m_type);
    }
}

void EventDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(!m_eventCtrl->GetValue().IsEmpty())
    {
        if(m_editing)
        {
            wxDateTime date = CreateDateTime();
            if(date > wxDateTime::Now() && m_reminder < 0)
                m_reminder = 0;
            m_db->EditEvent(m_id, m_eventCtrl->GetValue(), m_descriptionCtrl->GetValue(), date, m_reminder);
            if(m_type)
            {
                if(m_db->HasRecurrence(m_id)) m_db->EditRecurrence(m_id, m_type, m_days, m_months);
                else m_db->AddRecurrence(m_type, m_days, m_months, m_id);
            }
            else
                m_db->RemoveRecurrence(m_id);
        }
        else
        {
            m_id = m_db->AddEvent(m_eventCtrl->GetValue(), m_descriptionCtrl->GetValue(), CreateDateTime(), m_reminder).GetID();
            if(m_type) m_db->AddRecurrence(m_type, m_days, m_months, m_id);
        }
        EndModal(wxID_OK);
        return;
    }
    EndModal(wxID_CANCEL);
}

void EventDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void EventDialog::OnReminder(wxCommandEvent& event)
{
    int id = event.GetInt();
    switch(id)
    {
        case 0: m_reminder = 0; break;
        case 1: m_reminder = 5; break;
        case 2: m_reminder = 15; break;
        case 3: m_reminder = 60; break;
        case 4: m_reminder = 120; break;
        case 5: m_reminder = 1440; break;
        case 6: m_reminder = 2880; break;
        default:
        {
            ReminderDialog dialog(this, m_reminder);
            if(dialog.ShowModal() == wxID_OK)
            {
                m_reminder = dialog.GetReminder();
                long mins = m_reminder.ToLong();
                long days = mins/(60*24);
                mins -= days*60*24;
                long hours = mins/60;
                mins -= hours*60;
                m_reminderCtrl->SetString(7, wxString::Format(_("Own (%ld days, %ld hours, %ld minutes)"), days, hours, mins));
                m_reminderCtrl->SetSelection(7);
            }
            break;
        }
    }
}

void EventDialog::OnRecurrence(wxCommandEvent& event)
{
    m_type = event.GetInt();
    switch(m_type)
    {
        case 0: m_days = 0; m_months = 0; break;
        case 1: m_days = 1; m_months = 0; break;
        case 2: m_days = 7; m_months = 0; break;
        case 3: m_days = 14; m_months = 0; break;
        case 4: m_days = 0; m_months = 1; break;
        default:
        {
            RecurrenceDialog dialog(this, m_days, m_months);
            if(dialog.ShowModal() == wxID_OK)
            {
                m_days = dialog.GetDays();
                m_months = dialog.GetMonths();
                m_recurrenceCtrl->SetString(5, wxString::Format(_("Own (%d months, %d days)"), m_months, m_days));
                m_recurrenceCtrl->SetSelection(5);
            }
            break;
        }
    }
}

wxDateTime EventDialog::CreateDateTime()
{
    wxDateTime date = m_dateCtrl->GetValue();
    date.SetHour(m_timeCtrl->GetValue().GetHour());
    date.SetMinute(m_timeCtrl->GetValue().GetMinute());
    date.SetSecond(0);
    return date;
}

wxDateTime EventDialog::RoundTime()
{
    wxDateTime date = wxDateTime::Now();
    unsigned short hours = date.GetHour();
    unsigned short mins = date.GetMinute();
    if(mins+5 < 15)
        mins = 15;
    else if(mins+5 < 30)
        mins = 30;
    else if(mins+5 < 45)
        mins = 45;
    else if(mins+5 < 60)
    {
        hours++;
        mins = 0;
    }
    else
    {
        hours++;
        mins = 15;
    }
    date.SetHour(hours);
    date.SetMinute(mins);
    date.SetSecond(0);
    return date;
}
