/***************************************************************
 * Name:      eventdialog.h
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#ifndef EVENTDIALOG_H
#define EVENTDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/datectrl.h>
#include <wx/spinctrl.h>
#include <wx/timectrl.h>
#include "data/engine.h"

class ReminderDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(ReminderDialog)
public:
    ReminderDialog() {}
    ReminderDialog(wxWindow *parent, wxLongLong reminder);

    wxLongLong GetReminder();
private:
    wxSpinCtrl *m_days, *m_hours, *m_mins;
};

class RecurrenceDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(RecurrenceDialog)
public:
    RecurrenceDialog() {}
    RecurrenceDialog(wxWindow *parent, int days, int months);

    int GetDays();
    int GetMonths();
private:
    wxSpinCtrl *m_days, *m_moths;
};

class EventDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(EventDialog)
    DECLARE_EVENT_TABLE()
public:
    EventDialog() : m_db(), m_editing(false) {}
    EventDialog(wxWindow *parent, Engine *db);
    EventDialog(wxWindow *parent, Engine *db, Event event);
private:
    bool m_editing;
    Engine *m_db;
    long m_id;
    wxTextCtrl *m_eventCtrl, *m_descriptionCtrl;
    wxTimePickerCtrl *m_timeCtrl;
    wxDatePickerCtrl *m_dateCtrl;
    wxStaticText *m_reminderText;
    wxChoice *m_reminderCtrl, *m_recurrenceCtrl;
    wxLongLong m_reminder;
    wxBoxSizer *m_mainSizer;
    int m_type, m_days, m_months;

    void CreateControls();
    void FillControls(Event event);
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnReminder(wxCommandEvent& event);
    void OnRecurrence(wxCommandEvent& event);
    wxDateTime CreateDateTime();
    wxDateTime RoundTime();
};

#endif  /* EVENTDIALOG_H */

