/***************************************************************
 * Name:      eventmsgdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "dxdefs.h"
#include "eventmsgdialog.h"
#include "eventdialog.h"

IMPLEMENT_DYNAMIC_CLASS(LaterDialog, wxDialog)

BEGIN_EVENT_TABLE(LaterDialog, wxDialog)
    EVT_CHOICE(ID_REMINDERCHOICE, LaterDialog::OnReminder)
END_EVENT_TABLE()

LaterDialog::LaterDialog(wxWindow* parent, wxDateTime date)
: wxDialog(parent, wxID_ANY, wxString(_("Reminder time"))), m_date(date)
{
    RecalculateReminder(5);

    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Reminder:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    wxArrayString choices;
    choices.Add(_("5 minutes after"));
    choices.Add(_("10 minutes after"));
    choices.Add(_("15 minutes after"));
    choices.Add(_("Own"));
    m_reminderCtrl = new wxChoice(this, ID_REMINDERCHOICE, wxDefaultPosition, wxDefaultSize, choices);
    m_reminderCtrl->SetSelection(0);
    gridSizer->Add(m_reminderCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(mainSizer);
    mainSizer->Fit(this);
    mainSizer->SetSizeHints(this);
}

void LaterDialog::OnReminder(wxCommandEvent& event)
{
    int id = event.GetInt();
    switch(id)
    {
        case 0: RecalculateReminder(5); break;
        case 1: RecalculateReminder(10); break;
        case 2: RecalculateReminder(15); break;
        default:
        {
            ReminderDialog dialog(this, m_reminder);
            if(dialog.ShowModal() == wxID_OK)
                RecalculateReminder(dialog.GetReminder());
            break;
        }
    }
}

void LaterDialog::RecalculateReminder(wxLongLong reminder)
{
    m_reminder = -(wxDateTime::Now().GetTicks() + reminder.ToLong()*60 - m_date.GetTicks())/60;
}

IMPLEMENT_DYNAMIC_CLASS(EventMsgDialog, wxDialog)

BEGIN_EVENT_TABLE(EventMsgDialog, wxDialog)
    EVT_BUTTON(ID_BTN_REMINDLATER, EventMsgDialog::OnRemindLater)
END_EVENT_TABLE()

EventMsgDialog::EventMsgDialog(wxWindow *parent, Engine *db, Event event)
: wxDialog(parent, wxID_ANY, wxString(_("Event"))), m_db(db), m_id(event.GetID()), m_date(event.GetDate())
{
    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);

    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Event:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, event.GetEvent()), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    if(!event.GetDescr().IsEmpty())
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Description:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
        gridSizer->Add(new wxStaticText(this, wxID_ANY, event.GetDescr()), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    }
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, event.GetDate().Format(_T("%x %X"))), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxEmptyString), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    gridSizer->Add(new wxButton(this, ID_BTN_REMINDLATER, _("Remind later")));
    mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(mainSizer);
    mainSizer->Fit(this);
    mainSizer->SetSizeHints(this);
}

void EventMsgDialog::OnRemindLater(wxCommandEvent& /*event*/)
{
    LaterDialog dialog(this, m_date);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_db->EditEvent(m_id, dialog.GetReminder());
    }
    EndModal(wxID_CANCEL);
}

